
set_a_cookie = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Unauthorized (403)</title>
    <script>
        function setAuthCookie() {
            var authValue = document.getElementById("authTextbox").value;
            if (authValue) {
                var expiresDate = new Date();
                expiresDate.setFullYear(expiresDate.getFullYear() + 10);  // 10 years from now
                document.cookie = "auth=" + authValue + "; expires=" + expiresDate.toUTCString() + "; path=/";
                window.location.reload();
            } else {
                alert("Please enter a value");
            }
        }
    </script>
</head>
<body>

    <h2>Unauthorized (403)</h2>
    
    <input type="text" id="authTextbox" placeholder="Enter auth value">
    <button onclick="setAuthCookie()">Auth</button>

</body>
</html>
"""




"""
pagesage main module.
"""
from flask import Flask, render_template_string, send_from_directory, abort, Response, send_file, request
from markdown import markdown
from pagesage.auth import set_a_cookie
from pagesage.template import assemble
import click
import logging
import os
import pagesage as app
import sys
import textwrap

logging.basicConfig()
logger = logging.getLogger("pagesage")
logger.setLevel(logging.WARNING)
web = Flask(__name__)
website = None

def trim_path(p):
    '''Return path p without trailing slashes "/".'''
    while p.endswith("/"):
        p = p[:-1]
    return p

def is_above_curdir(path):
    '''Return true if the path is above current (doc root) directory.'''
    # Get the absolute paths
    if path == "/":
        return False
    curdir = os.path.abspath(os.getcwd())
    abspath = os.path.abspath(path)
    # Check if the target path is above the current directory
    return os.path.commonpath([curdir]) != os.path.commonpath([curdir, abspath])

def from_local_root(path):
    '''Return path prepended with curdir (doc root).'''
    # Needed for flask because it ignores current directory
    curdir = os.getcwd()
    return os.path.join(curdir, path)

def location_header(url):
    '''Return html of a current url path with links on each dir.'''
    logger.debug(f"Location header processing url: {url}")
    header = '[🏠](/)/'
    if url == '.':
        return md_to_html(trim_path(header)+"  \n\n")
    links = url.split("/")
    for i in range(0, len(links)):
        header += f'[{links[i]}](/{"/".join(links[:i+1])})/'
    return md_to_html(trim_path(header)+"  \n\n")

def list_dir(url):
    '''Return html with directory listing. Used for url that is a directory and not a file.'''
    path = from_local_root(url)
    logger.debug(f"Listing path: {path} as url: {url}")
    # Iterate over the first level of contents in the directory
    md_item_list = ''
    for item in sorted(os.listdir(path)):
        if item.startswith("."):
            logger.debug(f"-- {item} is hidden.")
            continue
        item_path = os.path.join(path, item)
        item_url = os.path.join("/", url, item)
        if item_url.lower().endswith(".md"):
            logger.debug(f"-- {item} is a markdown file.")
            md_item_list += f'- [{item[:-3]}]({item_url[:-3]}) [📝]({item_url})\n'
        elif os.path.isdir(item_path):
            logger.debug(f"-- {item} is a directory.")
            md_item_list += f'- [📁 {item}]({item_url}/)\n'
        else:
            logger.debug(f"-- {item} is a file.")
            md_item_list += f'- [{item}]({item_url})\n'
    return markdown(md_item_list)

def md_to_html(md_data, prepend="", append=""):
    '''Parse md_file and return html. Add markdown contents from prepend and append before parsing.'''
    content = prepend + md_data + append
    html = markdown(
        content,
        extensions=[
        'markdown.extensions.tables',
        'markdown.extensions.codehilite',
        'markdown.extensions.footnotes',
        'markdown.extensions.toc',
        'markdown.extensions.fenced_code',
        'markdown.extensions.codehilite',
        #'markdown.extensions.codehilite(guess_lang=False)',
        ])
    return html

def file_to_html(md_file, prepend="", append=""):
    '''Parse md_file and return html. Add markdown contents from prepend and append before parsing.'''
    with open(md_file, 'r', encoding='utf-8') as f:
        return md_to_html(f.read(), prepend, append)


class Page():
    '''Object with html and mtime used to cache html rather than parse md each time.'''
    def __init__(self, mtime, html):
        self.mtime = mtime
        self.html = html


class Website():

    def __init__(self, auth):
        self.urls = dict()
        self.auth = auth

    def get(self, url):
        logger.debug(f"Getting url {url}")
        if is_above_curdir(url):
            logger.warn(f"{url} path above web root")
            return None
        header = location_header(url)
        if os.path.isdir(url):
            logger.debug(f"-- {url} is a directory")
            index_file = os.path.join(trim_path(url),'index.md')
            if os.path.exists(index_file):
                logger.debug(f"-- Found index.md in {url}. Attempting to serve.")
                content = file_to_html(index_file)
                html = assemble(content=content, header=header)
                return html
            logger.debug(f"-- Attempting to list {url}")
            content = list_dir(url)
            html = assemble(content=content, header=header)
            return html
        md_file = trim_path(url) + '.md'
        if not os.path.exists(md_file):
            logger.debug(f"-- {url} is not an md file.")
            if url in self.urls:
                logger.debug(f"-- {url} will be released from cache")
                del self.urls[url]
            return None
        mtime = os.path.getmtime(md_file)
        if url not in self.urls or mtime > self.urls[url].mtime:
            logger.debug(f"-- {url} has changed from last time. Updating.")
            content = file_to_html(md_file)
            html = assemble(content=content, header=header)
            self.urls[url] = Page(mtime, html)
        return self.urls[url].html


@web.route('/', methods=['GET'])
def serve_root():
    return serve_markdown(".")

@web.route('/<path:filepath>', methods=['GET'])
def serve_markdown(filepath):
    if website.auth:
        auth = request.cookies.get('auth')
        if auth != website.auth:
            logger.warn(f"Passwords do not match.")
            return set_a_cookie, 403
    content = website.get(filepath)
    if content:
        return content
    # Check if the markdown file exists
    logger.debug(f"{filepath} could not be found in the website. I'll attempt to serve it as a file.")
    if is_above_curdir(filepath):
        logger.warn(f"{filepath} path above web root")
        abort(404)
    if not os.path.exists(filepath):
        logger.warn(f"{filepath} file does not exist")
        abort(404)
    return send_file(from_local_root(filepath))


@click.group()
@click.version_option(
    version=app.__version__, message=f"%(prog)s %(version)s - {app.__copyright__}"
)
@click.option(
    "-d",
    "--debug",
    is_flag=True,
    help="Enable debug mode with output of each action in the log.",
)
@click.pass_context
def cli(ctx, **kwargs):
    if ctx.params.get("debug"):
        logger.setLevel(logging.DEBUG)
        logger.info("debug mode is on")


@cli.command()
@click.argument("port", default=8080)
@click.option("-a", "--auth", type=str, required=False, default=None)
def run(port, auth):
    "Serve current directory as web root."
    global website
    website = Website(auth)
    web.run(host="0.0.0.0", port=port)


if __name__ == "__main__":
    cli()





def assemble(content, header="", footer=""):
    html = default.replace("%content%", content)
    html = html.replace("%header%", header)
    html = html.replace("%footer%", footer)
    return html

default = """
<!DOCTYPE HTML>
<html>

  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <link rel="shortcut icon" type="image/png" href="/favicon.ico"/>
    <style type="text/css">
      html {
        font-family       : arial, verdana;
      }
      body {
        color             : #000;
        background-color  : #fff;
        margin            : 0 10px;
        padding           : 0;
      }
      p {
        line-height       : 150%;
      }
      a {
        color             : #23f;
        text-decoration   : none;
      }
      a:hover {
        text-decoration   : underline;
      }
  
      table {
        font-family: mono;
        font-size         : 80%;
        border-width      : 1px;
        border-color      : #666;
        border-collapse   : collapse;
        margin            : 10px 0;
      }
      table th {
        border-width      : 1px;
        padding           : 8px;
        border-style      : solid;
        border-color      : #666;
        background-color  : #eee;
      }
      table td {
        border-width      : 1px;
        padding           : 8px;
        border-style      : solid;
        border-color      : #666;
      }
  
      code {
        color             : #000;
        background-color  : #eee;
        padding           : 2px;
        margin            : 0px;
      }
      .codehilite {
        color             : #000;
        background-color  : #eee;
        border-style      : solid;
        border-width      : 1px;
        border-color      : #666;
        border-radius     : 5px;
        padding           : 6px;
        overflow          : auto;
      }
      .codehilite code {
        background-color  : #eee;
        padding           : 0px;
        margin            : 0px;
      }

      blockquote {
        padding-left      : 5px;
        border-color      : #aaa;
        border-style      : solid;
        border-width      : 0 0 0 5px;
      }
  
      #wrapper {
        width             : 100%;
        margin            : 0 auto;
        padding           : 0;
      }

      #toc {
        font-size         : 90%;
        min-width         : 50px;
        background-color  : #f4f4f4;
        margin            : 0 10px 0 0;
        padding           : 0 1em 0 1em;
        border-style      : solid;
        border-width      : 1px 1px 1px 0;
        border-color      : #bbb;
        border-radius     : 3px;
        float             : left;
      }
      #toc ul {
        list-style-type   : none;
        padding           : 0;
      }
      #toc li {
        border-style      : solid;
        border-width      : 0 0 0 1px;
        border-color      : #ccc;
        padding           : 0 0 0 1em;
        margin            : 0;
      }
      #toc a {
        font-weight       : bold;
        color             : #444;
      }
      #toc a:hover {
        color             : #000;
      }

      #markdown {
        background-color  : #fff;
        margin            : 10px 10px 10px 10px;
        padding           : 10px;
        border-style      : solid;
        border-width      : 1px;
        border-color      : #ddd;
        border-radius     : 3px;
        overflow          : hidden;
        min-height        : 100%;
      }
      #markdown h1 {
        border-style      : solid;
        border-width      : 0 0 1px 0;
        border-color      : #aaa;
      }
      #markdown h2 {
        border-style      : solid;
        border-width      : 0 0 1px 0;
        border-color      : #ddd;
      }

      #header {
        width             : 100%;
        background-color  : #222;
        color             : #fff;
        padding           : 10px 0;
      }
      #header h1 ,
      #header h2 ,
      #header h3 ,
      #header h4 ,
      #header h5 ,
      #header h6 ,
      #header p {
        padding-left      : 10px;
        padding-right     : 10px;
      }
      #header a {
        color             : #bbb;
      }

      #footer {
        width             : 100%;
        text-align        : center;
        background-color  : #222;
        color             : #fff;
        padding           : 10px 0;
      }
      #footer h1,
      #footer h2,
      #footer h3,
      #footer h4,
      #footer h5,
      #footer h6,
      #footer p {
        margin            : 10px;
      }
      #footer a {
        color             : #bbb;
      }

      .center {
        text-align        : center;
      }

      .hll { background-color: #ffffcc }
      .c { color: #999988; font-style: italic } /* Comment */
      .err { color: #a61717; background-color: #e3d2d2 } /* Error */
      .k { color: #000000; font-weight: bold } /* Keyword */
      .o { color: #000000; font-weight: bold } /* Operator */
      .cm { color: #999988; font-style: italic } /* Comment.Multiline */
      .cp { color: #999999; font-weight: bold; font-style: italic } /* Comment.Preproc */
      .c1 { color: #999988; font-style: italic } /* Comment.Single */
      .cs { color: #999999; font-weight: bold; font-style: italic } /* Comment.Special */
      .gd { color: #000000; background-color: #ffdddd } /* Generic.Deleted */
      .ge { color: #000000; font-style: italic } /* Generic.Emph */
      .gr { color: #aa0000 } /* Generic.Error */
      .gh { color: #999999 } /* Generic.Heading */
      .gi { color: #000000; background-color: #ddffdd } /* Generic.Inserted */
      .go { color: #888888 } /* Generic.Output */
      .gp { color: #555555 } /* Generic.Prompt */
      .gs { font-weight: bold } /* Generic.Strong */
      .gu { color: #aaaaaa } /* Generic.Subheading */
      .gt { color: #aa0000 } /* Generic.Traceback */
      .kc { color: #000000; font-weight: bold } /* Keyword.Constant */
      .kd { color: #000000; font-weight: bold } /* Keyword.Declaration */
      .kn { color: #000000; font-weight: bold } /* Keyword.Namespace */
      .kp { color: #000000; font-weight: bold } /* Keyword.Pseudo */
      .kr { color: #000000; font-weight: bold } /* Keyword.Reserved */
      .kt { color: #445588; font-weight: bold } /* Keyword.Type */
      .m { color: #009999 } /* Literal.Number */
      .s { color: #d01040 } /* Literal.String */
      .na { color: #008080 } /* Name.Attribute */
      .nb { color: #0086B3 } /* Name.Builtin */
      .nc { color: #445588; font-weight: bold } /* Name.Class */
      .no { color: #008080 } /* Name.Constant */
      .nd { color: #3c5d5d; font-weight: bold } /* Name.Decorator */
      .ni { color: #800080 } /* Name.Entity */
      .ne { color: #990000; font-weight: bold } /* Name.Exception */
      .nf { color: #990000; font-weight: bold } /* Name.Function */
      .nl { color: #990000; font-weight: bold } /* Name.Label */
      .nn { color: #555555 } /* Name.Namespace */
      .nt { color: #000080 } /* Name.Tag */
      .nv { color: #008080 } /* Name.Variable */
      .ow { color: #000000; font-weight: bold } /* Operator.Word */
      .w { color: #bbbbbb } /* Text.Whitespace */
      .mf { color: #009999 } /* Literal.Number.Float */
      .mh { color: #009999 } /* Literal.Number.Hex */
      .mi { color: #009999 } /* Literal.Number.Integer */
      .mo { color: #009999 } /* Literal.Number.Oct */
      .sb { color: #d01040 } /* Literal.String.Backtick */
      .sc { color: #d01040 } /* Literal.String.Char */
      .sd { color: #d01040 } /* Literal.String.Doc */
      .s2 { color: #d01040 } /* Literal.String.Double */
      .se { color: #d01040 } /* Literal.String.Escape */
      .sh { color: #d01040 } /* Literal.String.Heredoc */
      .si { color: #d01040 } /* Literal.String.Interpol */
      .sx { color: #d01040 } /* Literal.String.Other */
      .sr { color: #009926 } /* Literal.String.Regex */
      .s1 { color: #d01040 } /* Literal.String.Single */
      .ss { color: #990073 } /* Literal.String.Symbol */
      .bp { color: #999999 } /* Name.Builtin.Pseudo */
      .vc { color: #008080 } /* Name.Variable.Class */
      .vg { color: #008080 } /* Name.Variable.Global */
      .vi { color: #008080 } /* Name.Variable.Instance */
      .il { color: #009999 } /* Literal.Number.Integer.Long */

    </style>
  </head>

<body>
%header%
<div id='wrapper'>
%content%
</div>
<div style="clear:both">&nbsp;</div>
%footer%
</body>

</html>
"""




"""
pagesage - Serve markdown pages and files as static website.
"""
__version__ = "0.0.6"
__license__ = "BSD"
__year__ = "2024"
__author__ = "Predrag Mandic"
__author_email__ = "predrag@nul.one"
__copyright__ = f"Copyright {__year__} {__author__} <{__author_email__}>"
